# -*- coding: utf-8 -*-
"""Test the apexpy.helper submodule

Notes
-----
Whenever function outputs are tested against hard-coded numbers, the test
results (numbers) were obtained by running the code that is tested.  Therefore,
these tests below only check that nothing changes when refactoring, etc., and
not if the results are actually correct.

These results are expected to change when IGRF is updated.

"""

import datetime as dt
import numpy as np
import pytest

from apexpy import helpers


def datetime64_to_datetime(dt64):
    """Convert numpy datetime64 object to a datetime datetime object.

    Parameters
    ----------
    dt64 : np.datetime64
        Numpy datetime64 object

    Returns
    -------
    dt.datetime
        Equivalent datetime object with a resolution of days

    Notes
    -----
    Works outside 32 bit int second range of 1970

    """
    year_floor = dt64.astype('datetime64[Y]')
    month_floor = dt64.astype('datetime64[M]')
    day_floor = dt64.astype('datetime64[D]')
    year = year_floor.astype(int) + 1970
    month = (month_floor
             - year_floor).astype('timedelta64[M]').astype(int) + 1
    day = (day_floor - month_floor).astype('timedelta64[D]').astype(int) + 1
    return dt.datetime(year, month, day)


class TestHelpers(object):
    """Test class for the helper sub-module."""

    def setup_method(self):
        """Set up a clean test environment."""
        self.in_shape = None
        self.calc_val = None
        self.test_val = None

    def teardown_method(self):
        """Clean up the test environment."""
        del self.in_shape, self.calc_val, self.test_val

    def eval_output(self, rtol=1e-7, atol=0.0):
        """Evaluate the values and shape of the calculated and expected output.
        """
        np.testing.assert_allclose(self.calc_val, self.test_val, rtol=rtol,
                                   atol=atol)
        assert np.asarray(self.calc_val).shape == self.in_shape
        return

    @pytest.mark.parametrize('lat', [90, 0, -90, np.nan])
    def test_checklat_scalar(self, lat):
        """Test good latitude check with scalars.

        Parameters
        ----------
        lat : int or float
            Latitude in degrees N

        """
        self.calc_val = helpers.checklat(lat)

        if np.isnan(lat):
            assert np.isnan(self.calc_val)
        else:
            assert self.calc_val == lat
        return

    @pytest.mark.parametrize('lat', [(90 + 1e-5), (-90 - 1e-5)])
    def test_checklat_scalar_clip(self, lat):
        """Test good latitude check with scalars just beyond the lat limits.

        Parameters
        ----------
        lat : int or float
            Latitude in degrees N

        """
        self.calc_val = helpers.checklat(lat)
        self.test_val = np.sign(lat) * np.floor(abs(lat))
        assert self.calc_val == self.test_val
        return

    @pytest.mark.parametrize('in_args,msg',
                             [([90 + 1e-4], "lat must be in"),
                              ([-90 - 1e-4, 'glat'], "glat must be in"),
                              ([[-90 - 1e-5, -90, 0, 90, 90 + 1e-4], 'glat'],
                               "glat must be in"),
                              ([[-90 - 1e-4, -90, np.nan, np.nan, 90 + 1e-5]],
                               'lat must be in')])
    def test_checklat_error(self, in_args, msg):
        """Test bad latitude raises ValueError with appropriate message.

        Parameters
        ----------
        in_args : list
            List of input arguments
        msg : str
            Expected error message

        """
        with pytest.raises(ValueError) as verr:
            helpers.checklat(*in_args)

        assert str(verr.value).startswith(msg)
        return

    @pytest.mark.parametrize('lat,test_lat',
                             [(np.linspace(-90 - 1e-5, 90 + 1e-5, 3),
                               [-90, 0, 90]),
                              (np.linspace(-90, 90, 3), [-90, 0, 90]),
                              ([-90 - 1e-5, 0, 90, np.nan],
                               [-90, 0, 90, np.nan]),
                              ([[-90, 0], [0, 90]], [[-90, 0], [0, 90]]),
                              ([[-90], [0], [90]], [[-90], [0], [90]])])
    def test_checklat_array(self, lat, test_lat):
        """Test good latitude with finite values.

        Parameters
        ----------
        lat : array-like
            Latitudes in degrees N
        test_lat : list-like
            Output latitudes in degrees N

        """
        self.calc_val = helpers.checklat(lat)
        self.in_shape = np.asarray(lat).shape
        self.test_val = test_lat
        self.eval_output(atol=1e-8)
        return

    @pytest.mark.parametrize('lat,test_sin', [
        (60, 0.96076892283052284), (10, 0.33257924500670238),
        ([60, 10], [0.96076892283052284, 0.33257924500670238]),
        ([[60, 10], [60, 10]], [[0.96076892283052284, 0.33257924500670238],
                                [0.96076892283052284, 0.33257924500670238]])])
    def test_getsinIm(self, lat, test_sin):
        """Test sin(Im) calculation for scalar and array inputs.

        Parameters
        ----------
        lat : float
            Latitude in degrees N
        test_sin : float
            Output value

        """
        self.calc_val = helpers.getsinIm(lat)
        self.in_shape = np.asarray(lat).shape
        self.test_val = test_sin
        self.eval_output()
        return

    @pytest.mark.parametrize('lat,test_cos', [
        (60, 0.27735009811261463), (10, 0.94307531289434765),
        ([60, 10], [0.27735009811261463, 0.94307531289434765]),
        ([[60, 10], [60, 10]], [[0.27735009811261463, 0.94307531289434765],
                                [0.27735009811261463, 0.94307531289434765]])])
    def test_getcosIm(self, lat, test_cos):
        """Test cos(Im) calculation for scalar and array inputs.

        Parameters
        ----------
        lat : float
            Latitude in degrees N
        test_cos : float
            Expected output

        """
        self.calc_val = helpers.getcosIm(lat)
        self.in_shape = np.asarray(lat).shape
        self.test_val = test_cos
        self.eval_output()
        return

    @pytest.mark.parametrize('in_time,year', [
        (dt.datetime(2001, 1, 1), 2001), (dt.date(2001, 1, 1), 2001),
        (dt.datetime(2002, 1, 1), 2002),
        (dt.datetime(2005, 2, 3, 4, 5, 6), 2005.090877283105),
        (dt.datetime(2005, 12, 11, 10, 9, 8), 2005.943624682902)])
    def test_toYearFraction(self, in_time, year):
        """Test the datetime to fractional year calculation.

        Parameters
        ----------
        in_time : dt.datetime or dt.date
            Input time in a datetime format
        year : int or float
            Output year with fractional values

        """
        self.calc_val = helpers.toYearFraction(in_time)
        np.testing.assert_allclose(self.calc_val, year)
        return

    @pytest.mark.parametrize('gc_lat,gd_lat', [
        (0, 0), (90, 90), (30, 30.166923849507356), (60, 60.166364190170931),
        ([0, 90, 30], [0, 90, 30.166923849507356]),
        ([[0, 30], [90, 60]], [[0, 30.16692384950735],
                               [90, 60.166364190170931]])])
    def test_gc2gdlat(self, gc_lat, gd_lat):
        """Test geocentric to geodetic calculation.

        Parameters
        ----------
        gc_lat : int or float
            Geocentric latitude in degrees N
        gd_lat : int or float
            Geodetic latitude in degrees N

        """
        self.calc_val = helpers.gc2gdlat(gc_lat)
        self.in_shape = np.asarray(gc_lat).shape
        self.test_val = gd_lat
        self.eval_output()
        return

    @pytest.mark.parametrize('in_time,test_loc', [
        (dt.datetime(2005, 2, 3, 4, 5, 6), (-16.505391672592904,
                                            122.17768157084515)),
        (dt.datetime(2010, 12, 11, 10, 9, 8), (-23.001554595838947,
                                               26.008999999955023)),
        (dt.datetime(2021, 11, 20, 12, 12, 12, 500000),
         (-19.79733856741465, -6.635177076865062)),
        (dt.datetime(1601, 1, 1, 0, 0, 0), (-23.06239721771427,
                                            -178.90131731228584)),
        (dt.datetime(2100, 12, 31, 23, 59, 59), (-23.021061422069053,
                                                 -179.23129780639425))])
    def test_subsol(self, in_time, test_loc):
        """Test the subsolar location calculation.

        Parameters
        ----------
        in_time : dt.datetime
            Input time
        test_loc : tuple
            Expected output

        """
        self.calc_val = helpers.subsol(in_time)
        np.testing.assert_allclose(self.calc_val, test_loc)
        return

    @pytest.mark.parametrize('in_time', [dt.datetime(1600, 12, 31, 23, 59, 59),
                                         dt.datetime(2101, 1, 1, 0, 0, 0)])
    def test_bad_subsol_date(self, in_time):
        """Test raises ValueError for bad time in subsolar calculation.

        Parameters
        ----------
        in_time : dt.datetime
            Input time

        """
        with pytest.raises(ValueError) as verr:
            helpers.subsol(in_time)

        assert str(verr.value).startswith('Year must be in')
        return

    @pytest.mark.parametrize('in_time', [None, 2015.0])
    def test_bad_subsol_input(self, in_time):
        """Test raises ValueError for bad input type in subsolar calculation.

        Parameters
        ----------
        in_time : NoneType or float
            Badly formatted input time

        """
        with pytest.raises(ValueError) as verr:
            helpers.subsol(in_time)

        assert str(verr.value).startswith('input must be datetime')
        return

    @pytest.mark.parametrize('in_dates', [
        np.arange(np.datetime64("2000"), np.datetime64("2001"),
                  np.timedelta64(1, 'M')).astype('datetime64[s]').reshape((3,
                                                                           4)),
        np.arange(np.datetime64("1601"), np.datetime64("2100"),
                  np.timedelta64(1, 'Y')).astype('datetime64[s]')])
    def test_subsol_datetime64_array(self, in_dates):
        """Verify subsolar point calculation using an array of np.datetime64.

        Parameters
        ----------
        in_time : array-like
            Array of input times

        Notes
        -----
        Tested by ensuring the array of np.datetime64 is equivalent to
        converting using single dt.datetime values

        """
        # Get the datetime64 output
        ss_out = helpers.subsol(in_dates)

        # Get the datetime scalar output for comparison
        self.in_shape = in_dates.shape
        true_out = [list(), list()]
        for in_date in in_dates.flatten():
            dtime = datetime64_to_datetime(in_date)
            out = helpers.subsol(dtime)
            true_out[0].append(out[0])
            true_out[1].append(out[1])

        # Evaluate the two outputs
        for i, self.calc_val in enumerate(ss_out):
            self.test_val = np.array(true_out[i]).reshape(self.in_shape)
            self.eval_output()

        return



# -*- coding: utf-8 -*-

import json
import os
import re

extensions = ['sphinx.ext.autodoc',
              'sphinx.ext.autosummary',
              'sphinx.ext.coverage',
              'sphinx.ext.viewcode',
              'sphinx.ext.githubpages',
              'sphinx.ext.napoleon',
              'sphinx.ext.extlinks',
              'autoapi.extension']

# Define common elements

source_suffix = '.rst'
master_doc = 'index'
project = 'ApexPy'
year = '2022'
zenodo = json.loads(open('../.zenodo.json').read())
author = ' and '.join([zcreator['name'] for zcreator in zenodo['creators']])
copyright = ', '.join([year, author])

# Get version number from __init__.py
regex = r"(?<=__version__..\s)\S+"
with open('../apexpy/__init__.py', 'r') as fin:
    text = fin.read()
match = re.findall(regex, text)
version = release = match[0].strip("'")

# Configure autoapi
autoapi_type = 'python'
autoapi_dirs = ['../apexpy']
autoapi_keep_files = True
autoapi_root = 'autoapi/generated'

# The name of an image file (relative to this directory) to place at the top
# of the sidebar.
html_logo = os.path.join(os.path.abspath('.'), 'apexpy.png')

# The theme to use for HTML and HTML Help pages.  See the documentation for
# a list of builtin themes.
html_theme = 'sphinx_rtd_theme'
html_theme_path = ["_themes", ]

pygments_style = 'trac'
templates_path = ['.']
html_use_smartypants = True
html_last_updated_fmt = '%b %d, %Y'
html_split_index = True
html_sidebars = {'**': ['searchbox.html', 'globaltoc.html', 'sourcelink.html']}
html_short_title = '-'.join([project, version])
autodoc_member_order = 'bysource'
autodoc_mock_imports = ['apexpy']
napoleon_use_ivar = True
napoleon_use_rtype = False
napoleon_use_param = False

extlinks = {'doi': ('http://dx.doi.org/%s', 'doi:%s')}



# -*- coding: utf-8 -*-

"""Entry point for the Command Line Interface"""

import argparse
import datetime as dt
import numpy as np
import sys

import apexpy

STDIN = sys.stdin.buffer
STDOUT = sys.stdout.buffer


def main():
    """Entry point for the script"""

    # Construct the description and parser for command-line arguments
    desc = 'Converts between geodetic, modified apex, quasi-dipole and MLT'
    parser = argparse.ArgumentParser(description=desc, prog='apexpy')

    parser.add_argument('source', metavar='SOURCE',
                        choices=['geo', 'apex', 'qd', 'mlt'],
                        help='Convert from {geo, apex, qd, mlt}')
    parser.add_argument('dest', metavar='DEST',
                        choices=['geo', 'apex', 'qd', 'mlt'],
                        help='Convert to {geo, apex, qd, mlt}')
    desc = ''.join(['YYYY[MM[DD[HHMMSS]]] date/time for IGRF coefficients, ',
                    'time part required for MLT calculations'])
    parser.add_argument('date', metavar='DATE', help=desc)
    parser.add_argument('--height', dest='height', default=0, metavar='HEIGHT',
                        type=float, help='height for conversion')
    parser.add_argument('--refh', dest='refh', metavar='REFH', type=float,
                        default=0,
                        help='reference height for modified apex coordinates')
    parser.add_argument('-i', '--input', dest='file_in', metavar='FILE_IN',
                        type=argparse.FileType('r'), default=STDIN,
                        help='input file (stdin if none specified)')
    parser.add_argument('-o', '--output', dest='file_out', metavar='FILE_OUT',
                        type=argparse.FileType('wb'), default=STDOUT,
                        help='output file (stdout if none specified)')

    # Get the command line arguements
    args = parser.parse_args()
    arg_array = np.loadtxt(args.file_in, ndmin=2)

    # Test the input arguments
    if 'mlt' in [args.source, args.dest] and len(args.date) < 14:
        desc = 'full date/time YYYYMMDDHHMMSS required for MLT calculations'
        raise ValueError(desc)
    if 9 <= len(args.date) and len(args.date) <= 13:
        desc = 'full date/time must be given as YYYYMMDDHHMMSS, not ' \
               + 'YYYYMMDDHHMMSS'[:len(args.date)]
        raise ValueError(desc)

    # Format the time input
    in_time = dt.datetime.strptime(args.date,
                                   '%Y%m%d%H%M%S'[:len(args.date) - 2])

    # Run the desired apex conversion
    apex_obj = apexpy.Apex(date=in_time, refh=args.refh)
    lats, lons = apex_obj.convert(arg_array[:, 0], arg_array[:, 1], args.source,
                                  args.dest, args.height, datetime=in_time)

    # Save the output to a file
    np.savetxt(args.file_out, np.column_stack((lats, lons)), fmt='%.8f')

    return


if __name__ == '__main__':
    sys.exit(main())



# -*- coding: utf-8 -*-
"""Unit tests for command line execution."""

import numpy as np
import os
import pytest
import subprocess


class TestCommandLine(object):
    """Test class for the command-line apexpy interface."""

    def setup_method(self):
        """Runs before every test method to create a clean environment."""
        # Define the desired working paths
        self.startdir = os.path.abspath(os.path.curdir)
        split_dirs = os.path.split(os.path.dirname(os.path.abspath(__file__)))
        self.workdir = split_dirs[0]

        # Change directory, if needed
        if self.startdir != self.workdir:
            os.chdir(self.workdir)

        # Define the test filenames
        self.outfile = os.path.join(split_dirs[1], 'output.txt')
        self.infile = os.path.join(split_dirs[1], 'test_convert.txt')
        self.singlefile = os.path.join(split_dirs[1],
                                       'test_convert_single_line.txt')

    def teardown_method(self):
        """Runs after every method to clean up previous testing."""
        # Remove testing output
        if os.path.isfile(self.outfile):
            os.remove(self.outfile)

        # Return to starting directory
        if self.startdir != os.path.abspath(os.path.curdir):
            os.chdir(self.startdir)

        del self.outfile, self.infile, self.singlefile

    def execute_command_line(self, command, command_kwargs=None,
                             pipe_out=False):
        """Execute the command and load data from self.outfile

        Parameters
        ----------
        command : list or str
            List or string containing command to execute using subprocess
        command_kwargs : dict or NoneType
            Dict containing optional kwargs for subprocess.Popen command or
            None if using defaults. (default=None)
        pipe_out : bool
            Return pipe output instead of output from a data file if True
            (default=False)

        Returns
        -------
        data : np.array, NoneType, or subprocess.Popen attribute
            Numpy array of data from output file, None if no file was created,
            or the requested output from the pipe command.

        """
        data = None

        if pipe_out:
            if command_kwargs is None:
                command_kwargs = {}

            pipe = subprocess.Popen(command, **command_kwargs)
            data = pipe.communicate()
            pipe.wait()
        else:
            os.system(" ".join(command))
            if os.path.isfile(self.outfile):
                data = np.loadtxt(self.outfile)

        return data

    @pytest.mark.parametrize("date_str", [("2015"), ("201501"), ('20150101'),
                                          ('20150101000000')])
    def test_convert_w_datetime(self, date_str):
        """Test command line with different date and time specification.

        Parameters
        ----------
        date_str : str
           Input date string

        """
        # Build and execute the apexpy command line call
        cmd = ['python', '-m', 'apexpy', 'geo', 'apex', date_str, '--height',
               '300', '-i', self.infile, '-o', self.outfile]
        data = self.execute_command_line(cmd)

        # Test the outfile existance and values
        assert data is not None, 'error executing: {:s}'.format(' '.join(cmd))
        np.testing.assert_allclose(data, [[57.47145462, 93.62657928],
                                          [58.52458191, 94.03150177],
                                          [59.57331467, 94.46398163]],
                                   rtol=1e-4)
        return

    def test_convert_single_line(self):
        """Test command line with a single line of output."""
        # Build and execute the apexpy command line call
        cmd = ['python', '-m', 'apexpy', 'geo', 'apex', '20150101000000',
               '--height', '300', '-i', self.singlefile, '-o', self.outfile]
        data = self.execute_command_line(cmd)

        # Test the outfile existance and values
        assert data is not None, 'error executing: {:s}'.format(' '.join(cmd))
        np.testing.assert_allclose(data, [57.47145462, 93.62657928], rtol=1e-4)
        return

    @pytest.mark.parametrize("height, out_list",
                             [("300", [57.47145462, 93.62657928]),
                              ("100 --refh=300", [56.01779556, 93.35305023])])
    def test_convert_stdin_stdout_w_height_flags(self, height, out_list):
        """Test use of pipe input to command-line call with height flags.

        Parameters
        ----------
        height : str
            String specifying height with command line options
        out_list : list
            List of expected output values

        """
        # Build and execute the apexpy command line call
        cmd = ''.join(['echo 60 15 | python -m apexpy geo apex 2015 --height ',
                       '{:s}'.format(height)])
        cmd_kwargs = {'shell': True, 'stdout': subprocess.PIPE}
        stdout, _ = self.execute_command_line(cmd, cmd_kwargs, True)

        assert stdout is not None, 'error executing: {:s}'.format(' '.join(cmd))
        np.testing.assert_allclose(np.array(stdout.split(b' '), dtype=float),
                                   out_list, rtol=1e-4)
        return

    def test_convert_mlt(self):
        """Test magnetic local time conversion."""
        # Build and execute the apexpy command line call
        cmd = ['python', '-m', 'apexpy', 'geo', 'mlt', '20150101000000',
               '--height', '300', '-i', self.singlefile, '-o', self.outfile]
        data = self.execute_command_line(cmd)

        # Test the outfile existance and values
        assert data is not None, 'error executing: {:s}'.format(' '.join(cmd))
        np.testing.assert_allclose(data, [57.469547, 1.06324], rtol=1e-4)
        return

    @pytest.mark.parametrize("date_str", [("201501010"), ("2015010100000")])
    def test_invalid_date(self, date_str):
        """Test raises ValueError with an invalid input date.

        Parameters
        ----------
        date_str : str
           Input date string

        """
        # Build and execute the command
        cmd = 'echo 60 15 | python -m apexpy geo apex {:s}'.format(date_str)
        cmd_kwargs = {'shell': True, 'stderr': subprocess.PIPE}
        _, stderr = self.execute_command_line(cmd, cmd_kwargs, True)

        # Evaluate the error output
        assert stderr is not None, 'error executing: {:s}'.format(' '.join(cmd))
        assert b'ValueError' in stderr, 'invalid date error not raised'
        return

    def test_mlt_nodatetime(self):
        """Test raises ValueError when time not provided for MLT calc."""
        # Build and execute the command
        cmd = 'echo 60 15 | python -m apexpy geo mlt 20150101'
        cmd_kwargs = {'shell': True, 'stderr': subprocess.PIPE}
        _, stderr = self.execute_command_line(cmd, cmd_kwargs, True)

        # Evaluate the error output
        assert stderr is not None, 'error executing: {:s}'.format(' '.join(cmd))
        assert b'ValueError' in stderr, 'invalid time error not raised'
        return

    @pytest.mark.parametrize("coords", [("foobar apex"), ("geo foobar")])
    def test_invalid_coord(self, coords):
        """Test raises error when bad coordinate input provided.

        Parameters
        ----------
        coords : str
           Input/output coordinate pairs

        """
        # Build and execute the command
        cmd = 'echo 60 15 | python -m apexpy {:s} 2015'.format(coords)
        cmd_kwargs = {'shell': True, 'stderr': subprocess.PIPE}
        _, stderr = self.execute_command_line(cmd, cmd_kwargs, True)

        # Evaluate the error output
        assert stderr is not None, 'error executing: {:s}'.format(' '.join(cmd))
        assert b'invalid choice' in stderr, 'invalid coord error not raised'
        return



# -*- coding: utf-8 -*-
"""Test the apexpy.fortranapex class

Notes
-----
Whenever function outputs are tested against hard-coded numbers, the test
results (numbers) were obtained by running the code that is tested.  Therefore,
these tests below only check that nothing changes when refactoring, etc., and
not if the results are actually correct.

These results are expected to change when IGRF is updated.

"""

from numpy.testing import assert_allclose
import os
import pytest

import apexpy
from apexpy import fortranapex as fa


class TestFortranApex(object):
    """Test class for the Python-wrapped Fortran functions."""

    def setup_method(self):
        """Initialize each test."""
        fa.loadapxsh(os.path.join(os.path.dirname(apexpy.__file__),
                                  'apexsh.dat'), 2000)

        # Set the inputs
        self.lat = 60
        self.lon = 15
        self.height = 100
        self.refh = 300
        self.vecflg = 1
        self.precision = 1e-10

        # Set the output values
        self.glat = 50.979549407958984
        self.glon = -66.16891479492188
        self.error = 2.8316469524725107e-06
        self.qlat = 56.531288146972656
        self.qlon = 94.1068344116211
        self.mlat = 55.94841766357422
        self.mlon = 94.1068344116211
        self.f1 = [1.079783, 0.10027137]
        self.f2 = [-0.24546318, 0.90718889]
        self.fmag = 1.0041800737380981
        self.d1 = [0.9495701, 0.25693053, 0.09049474]
        self.d2 = [0.10011087, -1.0780545, -0.33892432]
        self.d3 = [0.00865356, 0.27327004, -0.8666646]
        self.dmag = 1.100391149520874
        self.e1 = [1.0269295, 0.08382964, 0.03668632]
        self.e2 = [0.24740215, -0.82374191, -0.25726584]
        self.e3 = [0.01047826, 0.33089194, -1.04941]
        self.out = None
        self.test_out = None

    def teardown_method(self):
        """Clean environment after each test."""
        del self.lat, self.lon, self.height, self.refh, self.vecflg
        del self.qlat, self.qlon, self.mlat, self.mlon, self.f1, self.f2
        del self.fmag, self.d1, self.d2, self.d3, self.dmag, self.e1
        del self.e2, self.e3, self.precision, self.glat, self.glon, self.error
        del self.out, self.test_out

    def run_test_evaluation(self, rtol=1e-5, atol=1e-5):
        """Run the evaluation of the test results.

        Parameters
        ----------
        rtol : float
            Relative tolerance, default value based on old code's precision
            (default=1e-5)
        atol : float
            Absolute tolerance, default value based on old code's precision
            (default=1e-5)

        """

        assert self.out is not None, "No results to test"
        assert self.test_out is not None, "No 'truth' results provided"
        assert len(self.out) == len(self.test_out), "Mismatched outputs"

        for i, out_val in enumerate(self.out):
            assert_allclose(out_val, self.test_out[i], rtol=rtol, atol=atol)
        return

    def test_apxg2q(self):
        """Test fortran apex geographic to quasi-dipole."""
        # Get the output
        self.out = fa.apxg2q(self.lat, self.lon, self.height, self.vecflg)

        # Test the output
        self.test_out = (self.qlat, self.qlon, self.f1, self.f2, self.fmag)
        self.run_test_evaluation()
        return

    def test_apxg2all(self):
        """Test fortran apex geographic to all outputs."""
        # Get the output
        self.out = fa.apxg2all(self.lat, self.lon, self.height, self.refh,
                               self.vecflg)

        # Test the output
        self.test_out = (self.qlat, self.qlon, self.mlat, self.mlon, self.f1,
                         self.f2, self.fmag, self.d1, self.d2, self.d3,
                         self.dmag, self.e1, self.e2, self.e3)
        self.run_test_evaluation()
        return

    def test_apxq2g(self):
        """ Test fortran quasi-dipole to geographic."""
        # Get the output
        self.out = fa.apxq2g(self.lat, self.lon, self.height, self.precision)

        # Test the output
        self.test_out = (self.glat, self.glon, self.error)
        self.run_test_evaluation()
        return

    @pytest.mark.parametrize("lat", [0, 30, 60, 89])
    @pytest.mark.parametrize("lon", [-179, -90, 0, 90, 179])
    def test_g2q2d(self, lat, lon):
        """ Test fortran geographic to quasi-dipole and back again.

        Parameters
        ----------
        lat : int or float
            Latitude in degrees N
        lon : int or float
            Longitude in degrees E

        """
        self.out = fa.apxg2q(lat, lon, self.height, 0)
        self.test_out = fa.apxq2g(self.out[0], self.out[1], self.height,
                                  self.precision)

        # Test the results againt the initial input
        assert_allclose(self.test_out[0], lat, atol=0.01)
        assert_allclose(self.test_out[1], lon, atol=0.01)

    def test_apxq2g_lowprecision(self):
        """Test low precision error value."""
        self.out = fa.apxq2g(self.lat, self.lon, self.height, -1)

        # Test the output
        self.test_out = (51.00891876220703, -66.11973571777344, -9999.0)
        self.run_test_evaluation()
        return



#!/usr/bin/env python
# -*- encoding: utf-8 -*-

from glob import glob
from os import path, environ
from setuptools import setup, find_packages

# Include extensions only when not on readthedocs.org
if environ.get('READTHEDOCS', None) == 'True':
    extensions = []
else:
    from numpy.distutils.core import setup, Extension
    extensions = [
        Extension(name='apexpy.fortranapex',
                  sources=['fortranapex/igrf.f90',
                           'fortranapex/magfld.f90',
                           'fortranapex/apex.f90',
                           'fortranapex/makeapexsh.f90',
                           'fortranapex/apexsh.f90',
                           'fortranapex/checkapexsh.f90',
                           'fortranapex/fortranapex.pyf'])]

setup_kwargs = {'py_modules': [path.splitext(path.basename(pp))[0]
                               for pp in glob('*.py')],
                'ext_modules': extensions,
                'packages': find_packages()}

setup(**setup_kwargs)
